﻿//------------------------------------------------------------------------------------------
// <copyright file="MyListener.cs" company="seca gmbh &amp; co. kg">
//      Copyright © seca gmbh &amp; co. kg 2010-2011. All rights reserved.
// </copyright>
//------------------------------------------------------------------------------------------
using System;
using System.ComponentModel;
using System.Threading;
using MyModule.Properties;
using Seca.EmrContract;
using Seca.EmrContract.Measurements;
using System.Globalization;
using System.Diagnostics;

namespace MyModule
{
	/// <summary>
	/// Listens to messages from a electronic medical record (EMR) system
	/// </summary>
	internal class MyListener : BackgroundWorker
	{
		private readonly string[] firstNamesMale;
		private readonly string[] firstNamesFemale;
		private readonly string[] surnames;
		private readonly Random random = new Random();

		/// <summary>
		/// Initializes a new instance of the <see cref="MyListener"/> class.
		/// </summary>
		public MyListener()
		{
			var persistor = new MyPersistor();
			firstNamesMale = persistor.Data.FirstNamesMale.Split(';');
			firstNamesFemale = persistor.Data.FirstNamesFemale.Split(';');
			surnames = persistor.Data.Surnames.Split(';');
		}

		/// <summary>
		/// Gets or sets the measurement message.
		/// </summary>
		/// <value>The measurement message.</value>
		public string MeasurementMessage { get; set; }

		/// <summary>
		/// Runs listening code in a background thread.
		/// </summary>
		/// <param name="e">The relevant DoWorkEventArgs</param>
		protected override void OnDoWork(DoWorkEventArgs e)
		{
			Thread.CurrentThread.CurrentUICulture = CultureInfo.CurrentCulture;

			// Normally, patient data will be sent due to an event like pressing a "seca flash" button at the EMR.
			// Here we will wait some time before sending patient data automatically
			for (int i = 10; i > 0; i--)
			{
				var line2 = MeasurementMessage == null ? null : '\n' + MeasurementMessage;
				ReportProgress(
					-1,
					new MessageFromModuleEventArgs(string.Format(CultureInfo.InvariantCulture, Resources.MyListener_OnDoWork_New_data_in__0__seconds_1_, i, line2), null, TraceEventType.Information));
				Thread.Sleep(1000);
			}
			ReportProgress(
				-1,
				new MessageFromModuleEventArgs(MeasurementMessage, null, TraceEventType.Information));
			this.sendPatientFromEmr();
		}

		/// <summary>
		/// Retrieves patient information and passes it over to seca Flash.
		/// </summary>
		private void sendPatientFromEmr()
		{
			var patient = new Patient
								  {
									  Id = random.Next(100000, 999999).ToString(),
									  Gender = random.Next(2) == 0 ? Gender.Male : Gender.Female
								  };

			// write code here to retrieve patient information 
			// ...

			patient.FirstName =
				patient.Gender == Gender.Male ?
				firstNamesMale[random.Next(firstNamesMale.Length)] :
				firstNamesFemale[random.Next(firstNamesFemale.Length)];
			patient.Surname = surnames[random.Next(surnames.Length)];
			patient.DateOfBirth = new DateTime(random.Next(1910, 2011), random.Next(1, 13), random.Next(1, 28));

			// hand the patient info over to seca Flash
			var patientEventArgs = new PatientEventArgs(patient);
			ReportProgress(-1, patientEventArgs);

		}
	}
}
