﻿//------------------------------------------------------------------------------------------
// <copyright file="MySecaFlashModule.cs" company="seca gmbh &amp; co. kg">
//      Copyright © seca gmbh &amp; co. kg 2010-2011. All rights reserved.
// </copyright>
//------------------------------------------------------------------------------------------
using MyModule.Properties;
using Seca.EmrContract;

namespace MyModule
{
    /// <summary>
    /// A custom seca Flash module.
    /// </summary>
    /// <remarks>
    /// The EmrModule attribute also allows persistors to be null. In that case,
    /// flash will not show a configuration page.
    /// </remarks>
    [EmrModule("MyModule", "Protocol: MyModule | Version 1.0 | Vendor: MyCompany", typeof(MyPersistor))]
    public class MySecaFlashModule : Module
    {
        private MyListener listener;
        
        /// <summary>
        /// This method is called when the seca application starts listening for patient's data.
        /// </summary>
        /// <remarks>
        /// This method is called when the first subscription to <see cref="Module.PatientFromEmr"/>
        /// has been made.
        /// </remarks>
        /// <seealso cref="Module.PatientFromEmr"/>
        /// <seealso cref="StopListening"/>
        protected override void StartListening()
		{
            // instantiate a background process that listens for messages from the EMR system
			listener = new MyListener {WorkerReportsProgress = true, WorkerSupportsCancellation=true};
			listener.ProgressChanged += this.listenerProgressChanged;
            listener.RunWorkerAsync();
		}

        /// <summary>
        /// This method is called when the seca application stops listening for patient's data.
        /// </summary>
        /// This method is called when the last subscription to 
        /// <see cref="Module.PatientFromEmr"/>
        /// has been unsubsribed.
        /// <seealso cref="Module.PatientFromEmr"/>
        /// <seealso cref="StartListening"/>
        protected override void StopListening()
		{
			listener.ProgressChanged -= this.listenerProgressChanged;
			if (listener.IsBusy)
				listener.CancelAsync();
			listener = null;
		}

        /// <summary>
        /// Send progress information from the EMR listener to Flash.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event args of the event.</param>
        private void listenerProgressChanged(object sender, System.ComponentModel.ProgressChangedEventArgs e)
        {
            // send a message to Flash
            if (e.UserState is MessageFromModuleEventArgs)
            {
                var messageEventArgs = e.UserState as MessageFromModuleEventArgs;
                OnMessageFromModule(messageEventArgs);
            }

            // inform Flash about a patient received from the EMR system
            if (e.UserState is PatientEventArgs)
            {
                var patientEventArgs = e.UserState as PatientEventArgs;
                OnPatientFromEmr(patientEventArgs);
            }
        }
        
        /// <summary>
        /// Transports measurements from seca Flash to another system
        /// </summary>
        /// <param name="measurement">The measurement which shall be sent.</param>
		public override void SendMeasurement(Seca.EmrContract.Measurements.IMeasurement measurement)
        {
        	// pass the information over to another system
			if (listener != null)
			{
				if (listener.IsBusy)
					listener.CancelAsync();
				listener.MeasurementMessage = string.Format(
                                                            Resources.MySecaFlashModule_SendMeasurement_The_module_received___0_,
				                                            measurement);
				listener.RunWorkerAsync();
			}
        }
    }
}
